#include "GPAutomata.h"
#include "Random378.h"


int randomState() {
    return lrandMT() % stateCount;
}

int randomAction() {
    return lrandMT() % actionCount;
}


GPAutomata::GPAutomata() {
    _initialState = 0;
    _initialAction = 0;
    for (size_t i = 0; i < stateCount; ++i) {
        _states[i].transition[0] = 0;
        _states[i].transition[1] = 0;
        _states[i].action[0] = 0;
        _states[i].action[1] = 0;
    }
}


void GPAutomata::randomize() {
    _initialState = randomState();
    _initialAction = randomAction();
    for (size_t i = 0; i < stateCount; ++i) {
        _states[i].transition[0] = randomState();
        _states[i].transition[1] = randomState();
        _states[i].action[0] = randomAction();
        _states[i].action[1] = randomAction();
        _states[i].decider.randomize();
    }
}


void GPAutomata::mutate() {
    switch (lrandMT() % 10) {
        case 0: // change initial state
            _initialState = randomState();
            break;

        case 1: // change initial action
            _initialAction = randomAction();
            break;

        case 2: // change a transition
        case 3:
            _states[randomState()].transition[randomBit()] = randomState();
            break;

        case 4: // change an action
        case 5:
            _states[randomState()].action[randomBit()] = randomAction();
            break;

        case 6: // new decider
            _states[randomState()].decider.randomize();
            break;

        case 7: // cross two deciders
            ::crossover(
                _states[randomState()].decider,
                _states[randomState()].decider);
            break;

        case 8: // switch two deciders
            std::swap(
                _states[randomState()].decider,
                _states[randomState()].decider);
            break;

        case 9: // copy one decider over another
            _states[randomState()].decider = _states[randomState()].decider;
            break;
    }
}


void GPAutomata::crossover(GPAutomata& one, GPAutomata& two) {
    int p1 = randomState();
    int p2 = randomState();
    for (int i = p1; i != p2; i = (i + 1) % stateCount) {
        std::swap(one._states[i], two._states[i]);
    }
}


GPAutomata::Instance* GPAutomata::instance() const {
    return new Instance(this);
}
