#ifndef SW_GP_AUTOMATA_H
#define SW_GP_AUTOMATA_H


#include <stddef.h>
#include "ParseTree.h"


class GPAutomata {
public:
    // Create empty automata.
    GPAutomata();

    // Randomize this automata.
    void randomize();
    void mutate();
    static void crossover(GPAutomata& one, GPAutomata& two);

    class Instance {
    public:
        Instance(const GPAutomata* prototype) {
            _prototype = prototype;
            _state = _prototype->_initialState;
        }

        int initialAction() const {
            return _prototype->_initialAction;
        }

        int step(int* arguments) {
            const GPAutomata::State& state = _prototype->_states[_state];
            int result = state.decider.evaluate(arguments);
            int action;
            if (result % 2 == 0) {
                action = state.action[0];
                _state = state.transition[0];
            } else {
                action = state.action[1];
                _state = state.transition[1];
            }
            return action;
        }
        
    private:
        const GPAutomata* _prototype;
        int _state;
    };

    Instance* instance() const;

private:
    struct State {
        int transition[2];  // if-even, if-odd
        int action[2];      // if-even, if-odd
        ParseTree decider;
    };

    int _initialState;
    int _initialAction;
    State _states[stateCount];

    friend class Instance;
};


inline void crossover(GPAutomata& one, GPAutomata& two) {
    GPAutomata::crossover(one, two);
}


#endif
