#include <algorithm>
#include <fstream>
#include <iomanip>
#include <iostream>
#include <sstream>
#include "Constants.h"
#include "its.h"
#include "Random378.h"
#include "Submarine.h"
using namespace std;


void updateGeneration(Submarine* population);


float totalBattleTime = 0;
float totalAverageDistance = 0;
float totalMinDistance = 0;
float totalMaxDistance = 0;
int actionCounts[actionCount];
int shotCounts[weaponCount];
int hitCounts[weaponCount];


float ratio(int num, int den) {
    if (den == 0) {
        return 0;
    } else {
        return float(num) / den;
    }
}


int main() {
    cout << "Beginning Submarine Wars" << endl;

    SetNumv(inputCount);
    SetConstSize(100);
    SignedCon(1);
    seedMT(98765);

    for (size_t run = 0; run < runCount; ++run) {
        std::ostringstream os;
        os << "run" << run;
        std::ofstream weaponStats((os.str() + ".weapon.dat").c_str());
        std::ofstream actionStats((os.str() + ".action.dat").c_str());
        std::ofstream timeStats  ((os.str() + ".time.dat").c_str());
        std::ofstream hitStats   ((os.str() + ".hit.dat").c_str());

        cout << "Run: " << run << endl;
        Submarine population[populationSize];
        for (size_t i = 0; i < populationSize; ++i) {
            population[i].randomize();
        }

        for (size_t gen = 0; gen < generationCount; ++gen) {
            std::fill(actionCounts, actionCounts + actionCount, 0);
            totalBattleTime = 0;
            totalAverageDistance = 0;
            totalMinDistance = 0;
            totalMaxDistance = 0;
            std::fill(hitCounts, hitCounts + weaponCount, 0);
            std::fill(shotCounts, shotCounts + weaponCount, 0);


            updateGeneration(population);


            // Report statistics.

            cout << gen << "  ";

            // Report weapons.
            float weaponSums[3] = {0.0f, 0.0f, 0.0f};
            for (size_t i = 0; i < populationSize; ++i) {
                weaponSums[0] += population[i].getForward();
                weaponSums[1] += population[i].getAft();
                weaponSums[2] += population[i].getPulses();
            }
            const float total = weaponSums[0] + weaponSums[1] + weaponSums[2];
            weaponSums[0] /= total;
            weaponSums[1] /= total;
            weaponSums[2] /= total;
            cout << "W: " << setprecision(3)
                 << weaponSums[0] << ", "
                 << weaponSums[1] << ", "
                 << weaponSums[2];
            weaponStats << weaponSums[0] << "  " << weaponSums[1] << "  " << weaponSums[2] << endl;

            // Report actions.
            int totalActions = 0;
            for (size_t i = 0; i < actionCount; ++i) {
                totalActions += actionCounts[i];
            }

            cout << "  A:";
            for (size_t i = 0; i < actionCount; ++i) {
                cout << ", " << actionCounts[i] ;
                actionStats << float(actionCounts[i]) / totalActions << "  ";
            }
            actionStats << endl;

            const size_t battleCount = populationSize / 2;

            cout << "  T:" << (totalBattleTime / battleCount) << "  "
                 << (totalAverageDistance / battleCount) << "  "
                 << (totalMinDistance / battleCount) << "  "
                 << (totalMaxDistance / battleCount) << endl;



            timeStats << (totalBattleTime / battleCount) << "  "
                      << (totalAverageDistance / battleCount) << "  "
                      << (totalMinDistance / battleCount) << "  "
                      << (totalMaxDistance / battleCount) << endl;

            hitStats << ratio(hitCounts[0], shotCounts[0]) << "  "
                     << ratio(hitCounts[1], shotCounts[1]) << "  "
                     << ratio(hitCounts[2], shotCounts[2]) << endl;
        }
    }
}


void battlePair(Submarine& one, Submarine& two) {
    BattleResult br;
    for (int i = 0; i < 3; ++i) {
        br = battle(one, two);
        if (br.winner != BattleResult::TIE) {
            break;
        }
    }

    for (size_t i = 0; i < actionCount; ++i) {
        actionCounts[i] += br.actionCounts[i];
    }

    for (size_t i = 0; i < weaponCount; ++i) {
        shotCounts[i] += br.shotCounts[i];
        hitCounts[i]  += br.hitCounts[i];
    }

    totalBattleTime += br.time;
    if (br.distance.getCount()) {
        totalAverageDistance += br.distance.getAverage();
        totalMinDistance += br.distance.getMin();
        totalMaxDistance += br.distance.getMax();
    }

    if (br.winner == BattleResult::ONE) {
        two = one;
    } else if (br.winner == BattleResult::TWO) {
        one = two;
    } else {  // Mutate them both.
        one.mutate();
        two.mutate();
        if (randomBit()) {
            std::swap(one, two);
        }
    }
}


void updateGeneration(Submarine* population) {
    std::random_shuffle(population, population + populationSize, randMT);
    for (size_t i = 0; i < tournamentCount; ++i) {
        Submarine* tournament = population + i * tournamentSize;

        // Winner is copied over loser.
        battlePair(tournament[0], tournament[1]);
        battlePair(tournament[2], tournament[3]);

        crossover(tournament[1], tournament[3]);
        tournament[1].mutate();
        tournament[3].mutate();
    }
}
