#ifndef SW_SUBMARINE_H
#define SW_SUBMARINE_H


#include "Constants.h"
#include "GPAutomata.h"
#include "Statistics.h"


class SubmarineInstance;


struct BattleResult {
    enum Winner {
        ONE,
        TWO,
        TIE,
    };

    float time;
    Statistics distance;
    Winner winner;
    int actionCounts[actionCount];
    int shotCounts[weaponCount];
    int hitCounts[weaponCount];
};


class Submarine {
public:
    Submarine();

    void randomize();

    static BattleResult battle(Submarine& one, Submarine& two);
    static void crossover(Submarine& one, Submarine& two);
    void mutate();
    
    SubmarineInstance* instance() const;

    int getForward() const {
        int total = 0;
        for (size_t i = 0; i < totalWeapons; ++i) {
            total += (_weapons[i] == 'F');
        }
        return total;
    }

    int getAft() const {
        int total = 0;
        for (size_t i = 0; i < totalWeapons; ++i) {
            total += (_weapons[i] == 'A');
        }
        return total;
    }

    int getPulses() const {
        int total = 0;
        for (size_t i = 0; i < totalWeapons; ++i) {
            total += (_weapons[i] == 'P');
        }
        return total;
    }

private:
    char _weapons[totalWeapons];
    GPAutomata _behavior;
    
    friend class SubmarineInstance;
};


/// Returns true if the first submarine won.
inline BattleResult battle(Submarine& one, Submarine& two) {
    return Submarine::battle(one, two);
}

inline void crossover(Submarine& one, Submarine& two) {
    Submarine::crossover(one, two);
}


#endif
