#ifndef ITS_H
#define ITS_H

#include <ostream>

/*
 *
 *
 * Stack based integer parse tree routines
 *
 *
 *
 */


/*************************************************************
 Node values:
   0-255     terminals   x1, x2, ...
   256-511   unaries     ~, odd, com
   512-767   binaries    +,-,=,>=,<=,>,<,max,min
   768-1023  trinaries   ITE
   1024-1279 positive ephemeral constants
   1280+     negative ephemeral constants
*************************************************************/

/**********************/
/*                    */
/*  Begin Prototypes  */
/*                    */
/**********************/

//Routine to set the number of variables - this is the number of
//nonconstant terminals available
extern void SetNumv(int n);

//routine to set the maximum magnitude of constants in the GP-language
//The argument n forms an upper bound on the size of integer constants 
//permitted
extern void SetConstSize(int n);

//If argument is zero, constants are positive or zero, otherwise constants
//can also be negative 
extern void SignedCon(int n);

//Permits the use of a dollar-free LaTeX output format
extern void DollarLess(int p);  //patch for Latex < >

/***************I/O*********************************/
extern void Print(std::ostream &aus,int *n,int &p);
extern void Print(std::ostream &aus,int *n);

/**********Tree creation and deletion**************/
extern void RandTree(int *t,int n);   //generate a random tree
extern void CopyTree(int *t,int *n); //copy a tree

/**********Evaluation and counting routines********/
extern int SizeTree(const int *t);  //compute the size of a tree

extern int EvalTreeV(const int *t,const int *v,int n);//evaluate a multivariate tree
                                          //the second argument must be
                                          //an array with enough entries
                                          //tree has n nodes

extern int EvalTreeV(const int *t,const int *v);   //evaluate a multivariate tree
                                      //the second argument must be
                                      //an array with enough entries
/****************Genetic Routines******************/
//Return a random node in the range 0..k-1.  It is he users job to pass
//a reasonable value.  This routine exists mostly as a helper to

//Cross over a pair of trees at a random point
extern void Cross(int *n,   //First tree
                  int *m,   //Second tree
		  int mts   //Max tree size
          );

//Mutate a tree, replacing a subtree with a random subtree the same size
extern void Mutate(int *n);

extern void str2tree(int *t,char *s);  //convert a string to a tree

#endif
